"""
Test suites for the implementation of analyses.

Analyses replace and generalise the earlier notion of a "production" from v0.3 and earlier.
"""

import unittest

class TestBaseAnalysis(unittest.TestCase):
    pass

class TestSimpleAnalysis(TestBaseAnalysis):
    pass

class TestEventAnalysis(TestBaseAnalysis):
    pass

class TestProjectAnalysis(TestBaseAnalysis):
    pass
